# PySBOM
# Copyright (C) 2026-present Anton Lydell
# SPDX-License-Identifier: GPL-3.0-or-later

"""The core functionality of the CLI."""

# Standard library
import logging
from enum import StrEnum
from typing import NoReturn

# Third party
import click

logger = logging.getLogger(__name__)


class Color(StrEnum):
    """Terminal colors."""

    SUCCESS = 'green'
    WARNING = 'yellow'
    ERROR = 'red'


def exit_program(
    error: bool,
    ctx: click.Context | None = None,
    message: str | None = None,
    color: Color | None = None,
) -> NoReturn:
    """Exit the program with an exit code and an optional message.

    Parameters
    ----------
    error : bool
        True if an error occurred and the exit code will
        be 1 and False for success and the exit code 0.

    ctx : click.Context or None, default None
        The context of the program. If None the program will exit with
        :exc:`SystemExit` instead of :func:`click.Context.exit`.

    message : str or None, default None
        An optional message to print before exiting. If None no message is printed.

    color : pysbom.cli.core.Color or None, default None
        If specified it will override the default color of the `message`, which is
        red for error and green for success.
    """

    if error:
        exit_code = 1
        _color = Color.ERROR if color is None else color
        log_level = logging.ERROR
    else:
        exit_code = 0
        _color = Color.SUCCESS if color is None else color
        log_level = logging.INFO

    if message:
        click.secho(message=message, fg=_color)
        logger.log(level=log_level, msg=message)

    if ctx is not None:
        ctx.exit(code=exit_code)
    else:
        raise SystemExit(exit_code)


def echo_with_log(message: str, log_level: int = logging.INFO, color: Color | None = None) -> None:
    """Echo a message to the terminal and write it as a log statement.

    Parameters
    ----------
    log_level : int, default logging.INFO
        The log level to use.

    color : pysbom.cli.core.Color or None, default None
        The terminal foreground color to use for the terminal message.
    """

    click.secho(message=message, fg=color)
    logger.log(level=log_level, msg=message)



# PySBOM
# Copyright (C) 2026-present Anton Lydell
# SPDX-License-Identifier: GPL-3.0-or-later

"""Metadata about the PySBOM package."""

# Standard library
from datetime import date

__version_info__ = (0, 1, 0)
"""PySBOM version as a comparable tuple (MAJOR, MINOR, PATCH), following SemVer."""

__version__ = '.'.join(str(v) for v in __version_info__)
"""The PySBOM version string."""

__release_date__ = date(2026, 2, 14)
"""The release date of the current version."""



# PySBOM
# Copyright (C) 2026-present Anton Lydell
# SPDX-License-Identifier: GPL-3.0-or-later

"""The entry point of the sub-command `scan`."""

# Third party
import click


@click.command()
@click.pass_context
@click.argument('sources', nargs=-1, type=click.STRING)
def scan(ctx: click.Context, sources: tuple[str, ...] | None) -> None:
    """Discover Python environments and generate CycloneDX SBOMs

    \b
    Examples
    --------
    \b
    Use glob patterns from the configuration file:
        $ pysbom scan

    \b
    Provide glob patterns directly on the command line:
        $ pysbom scan "/opt/*/venv" "/opt/conda/envs/*" "/home/*/miniconda3/envs/*"

    \b
    Use recursive glob patterns:
        $ pysbom scan "/srv/**/venv"
    """



# PySBOM
# Copyright (C) 2026-present Anton Lydell
# SPDX-License-Identifier: GPL-3.0-or-later

"""PySBOM — Automated discovery and SBOM generation for Python environments

PySBOM (Python Software Bill of Materials) is a CLI tool that discovers Python
environments on servers and generates standardized SBOM files for each of them.
"""

# Local
from pysbom.metadata import (
    __release_date__,
    __version__,
    __version_info__,
)

# The Public API
__all__ = [
    # metadata
    '__release_date__',
    '__version__',
    '__version_info__',
]



# PySBOM
# Copyright (C) 2026-present Anton Lydell
# SPDX-License-Identifier: GPL-3.0-or-later

"""Execute the PySBOM CLI as `python -m pysbom`."""

# Local
from pysbom.cli.main import main

if __name__ == '__main__':
    main()



# PySBOM
# Copyright (C) 2026-present Anton Lydell
# SPDX-License-Identifier: GPL-3.0-or-later

"""The entry point of the PySBOM CLI."""

# Third party
import click

# Local
from pysbom.cli.scan import scan
from pysbom.metadata import __release_date__


@click.group(
    name='pysbom',
    context_settings={'help_option_names': ['-h', '--help'], 'max_content_width': 1000},
)
@click.version_option(
    message=(
        f'%(prog)s, version: %(version)s, release date: {__release_date__}, maintainer: Anton Lydell'
    )
)
@click.pass_context
def main(ctx: click.Context) -> None:
    """PySBOM - Python Software Bill of Materials (SBOM)"""


for cmd in (scan,):
    main.add_command(cmd)

if __name__ == '__main__':
    main()
